indexing
	description: "{
      A class resembling a hash table, but actually having a hash table
}"
	license: "Eiffel Forumn License v2 - See license.txt"
	source: "{
		Amalasoft Corporation
		273 Harwood Avenue
		Littleton, MA 01460 USA
		}"
	status: "Copyright 1995-2008, Amalasoft";
	date: "$Date: 2008/12/03 $";
	revision: "$Revision: 001$";
	howto: "{
    Create an instance of this class with a capacity
	 Use typical HASH_TABLE routines to add, delete and access items.
}"

class AEL_DS_INDIRECT_TABLE [G, H -> HASHABLE]

create
	make

--|========================================================================
feature {NONE} -- Initialization
--|========================================================================

	make (n: INTEGER) is
			-- Make a table with a capacity of at least 'n' items
		require
			valid_size: n >= 0
		do
			core_make
			make_table (n)
		end

	--|--------------------------------------------------------------

	core_make is
		do
		end

	--|--------------------------------------------------------------

	make_table (n: INTEGER) is
		require
			valid_size: n >= 0
		do
			create table.make (n)
		end

--|========================================================================
feature -- Status
--|========================================================================

	count: INTEGER is
			-- Number of items in table
		do
			Result := table.count
		end

	is_empty: BOOLEAN is
			-- Are there no items in the table?
		do
			Result := count = 0
		end

	--|--------------------------------------------------------------

	has (k: H): BOOLEAN is
			-- Does the table have an item with key 'k'?
		do
			Result := table.has (k)
		end

--|========================================================================
feature -- Access
--|========================================================================

	item (k: H): G is
			-- Item in table with key 'k'
		do
			Result := table.item (k)
		end

	--|--------------------------------------------------------------

	item_for_iteration: G is
			-- Item at current cursor position
		do
			Result := table.item_for_iteration
		end

--|========================================================================
feature -- Cursor movement and state
--|========================================================================

	cursor: CURSOR is do Result := table.cursor end
			-- Table cursor

	start is do table.start end
			-- Set table to start position

	forth is do table.forth end
			-- Advance table one position

	after: BOOLEAN is do Result := table.after end
			-- Is table after?

--|========================================================================
feature -- Element change
--|========================================================================

	extend (v: G; k: H) is
			-- Add item 'v' to table using key 'k'
		require
			is_new: not has (k)
		do
			table.extend (v, k)
		ensure
			added: has (k)
		end

	--|--------------------------------------------------------------

	force (v: G; k: H) is
			-- Add a new item, even if one exists with same key
		require
			exits: v /= Void
		do
			if has (k) then
				remove (k)
			end
			extend (v, k)
		end

	--|--------------------------------------------------------------

	put (v: G; k: H) is
			-- Add a new item if it's not already in the table
		do
			if not has (k) then
				extend (v, k)
			end
		end

--|========================================================================
feature -- Element removal
--|========================================================================

	wipe_out is
			-- Remove all items
		do
			table.wipe_out
		end

	--|--------------------------------------------------------------

	remove (k: H) is
			-- Remove item with given key
		require
			belongs: has (k)
		do
			table.remove (k)
		ensure
			removed: not has (k)
		end

--|========================================================================
feature {AEL_DS_INDIRECT_TABLE} -- Structure
--|========================================================================

	table: HASH_TABLE [G, H]
			-- Actual table
	--|--------------------------------------------------------------
invariant
	table_exists: table /= Void

end -- class AEL_DS_INDIRECT_TABLE
